@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
-import codecs
+import io
 import logging
-from .grammar import EMPTY, STOP
-from .tables import LALR, SLR, SHIFT, REDUCE, ACCEPT
-from .exceptions import ParseError, ParserInitError, DisambiguationError, \
-    DynamicDisambiguationConflict, SRConflicts, RRConflicts, \
-    expected_symbols_str
-from .common import Location, position_context, pos_to_line_col, ErrorContext
-from .actions import pass_none
-from .termui import prints, h_print, a_print
+from parglare.grammar import EMPTY, STOP
+from parglare.tables import LALR, SLR, SHIFT, REDUCE, ACCEPT
+from parglare.exceptions import (ParseError, ParserInitError,
+                                 DisambiguationError,
+                                 DynamicDisambiguationConflict, SRConflicts,
+                                 RRConflicts, expected_symbols_str)
+from parglare.common import Location, position_context, pos_to_line_col, ErrorContext
+from parglare.trees import NodeTerm, NodeNonTerm
+from parglare.actions import pass_none
+from parglare.termui import prints, h_print, a_print
 from parglare import termui
 
 
@@ -151,7 +153,7 @@ class Parser(object):
         Args:
             file_name(str): A file name.
         """
-        with codecs.open(file_name, 'r', 'utf-8') as f:
+        with io.open(file_name, 'r', encoding='utf-8') as f:
             content = f.read()
         return self.parse(content, file_name=file_name, **kwargs)
 
@@ -279,7 +281,7 @@ class Parser(object):
                 new_head = LRStackNode(
                     self,
                     state=act.state,
-                    shift_level=head.shift_level + 1,
+                    frontier=head.frontier + 1,
                     token=head.token_ahead,
                     layout_content=head.layout_content_ahead,
                     position=new_position,
@@ -312,7 +314,7 @@ class Parser(object):
                     new_head = LRStackNode(
                         self,
                         state=next_state,
-                        shift_level=head.shift_level,
+                        frontier=head.frontier,
                         position=head.position,
                         production=production,
                         start_position=start_reduction_head.start_position,
@@ -328,7 +330,7 @@ class Parser(object):
                     new_head = LRStackNode(
                         self,
                         state=next_state,
-                        shift_level=head.shift_level,
+                        frontier=head.frontier,
                         position=head.position,
                         production=production,
                         start_position=head.end_position,
@@ -362,7 +364,7 @@ class Parser(object):
         """
         def inner_call_actions(node):
             sem_action = node.symbol.action
-            if isinstance(node, NodeTerm):
+            if node.is_term():
                 if sem_action:
                     try:
                         result = sem_action(node.context, node.value,
@@ -478,9 +480,7 @@ class Parser(object):
         in_len = len(input_str)
         tokens = []
 
-        # add special tokens (EMPTY and STOP) if they are applicable
-        if EMPTY in actions:
-            tokens.append(EMPTY_token)
+        # add special STOP token if they are applicable
         if STOP in actions:
             if not self.consume_input \
                or (self.consume_input and position == in_len):
@@ -532,7 +532,7 @@ class Parser(object):
             if type(tok) is tuple:
                 tok, *additional_data = tok
             if tok:
-                tokens.append(Token(symbol, tok, additional_data))
+                tokens.append(Token(symbol, tok, position, additional_data))
                 if finish_flags[idx]:
                     break
         return tokens
@@ -555,7 +555,8 @@ class Parser(object):
                 if type(tok) is tuple:
                     tok, *additional_data = tok
                 if tok:
-                    tokens.append(Token(terminal, tok, additional_data))
+                    tokens.append(Token(terminal, tok, context.position,
+                                        additional_data))
         return tokens
 
     def _init_dynamic_disambiguation(self, context):
@@ -650,7 +651,7 @@ class Parser(object):
             if self.call_actions_during_tree_build and sem_action:
                 sem_action(context, token.value, *token.additional_data)
 
-            return NodeTerm(context)
+            return NodeTerm(context, token)
 
         if sem_action:
             result = sem_action(context, token.value, *token.additional_data)
@@ -685,7 +686,9 @@ class Parser(object):
                 h_print("Building non-terminal node",
                         "'{}'.".format(production.symbol.name), level=2)
 
-            bt_result = NodeNonTerm(context, children=subresults)
+            bt_result = NodeNonTerm(context, children=subresults,
+                                    production=production)
+            context.node = bt_result
             if not self.call_actions_during_tree_build:
                 return bt_result
 
@@ -750,10 +753,6 @@ class Parser(object):
         if len(tokens) <= 1:
             return tokens
 
-        # prefer STOP over EMPTY
-        if STOP_token in tokens:
-            tokens = [t for t in tokens if t != EMPTY_token]
-
         # Longest-match strategy.
         max_len = max((len(x.value) for x in tokens))
         tokens = [x for x in tokens if len(x.value) == max_len]
@@ -819,7 +818,7 @@ class Parser(object):
         while head.position < len(head.input_str):
             head.position += 1
             token = self._next_token(head)
-            if token and token is not EMPTY_token:
+            if token:
                 head.token_ahead = token
                 return True
         return False
@@ -844,18 +843,18 @@ class LRStackNode(object):
     An element of the LR parsing stack. Also the parsing context.
     """
 
-    __slots__ = ['parser', 'state', 'shift_level', 'position', 'results',
+    __slots__ = ['parser', 'state', 'frontier', 'position', 'results',
                  'start_position', 'end_position', 'token_ahead', 'token',
                  'production', 'layout_content', 'layout_content_ahead',
                  'node']
 
-    def __init__(self, parser, state, shift_level, position, results=None,
+    def __init__(self, parser, state, frontier, position, results=None,
                  start_position=None, end_position=None, token=None,
                  token_ahead=None, production=None, layout_content='',
                  layout_content_ahead=''):
         self.parser = parser
         self.state = state
-        self.shift_level = shift_level
+        self.frontier = frontier
         self.position = position
 
         self.results = results
@@ -906,110 +905,18 @@ class LRStackNode(object):
         self.parser.extra = new_value
 
 
-class Node(object):
-    """A node of the parse tree."""
-
-    __slots__ = ['context']
-
-    def __init__(self, context):
-        context.node = self
-        self.context = context
-
-    def __repr__(self):
-        return str(self)
-
-    def __iter__(self):
-        return iter([])
-
-    def __reversed__(self):
-        return iter([])
-
-    def __getattr__(self, name):
-        return getattr(self.context, name)
-
-
-class NodeNonTerm(Node):
-    __slots__ = ['production', 'children']
-
-    def __init__(self, context, children):
-        super(NodeNonTerm, self).__init__(context)
-        self.children = children
-
-    def tree_str(self, depth=0):
-        indent = '  ' * depth
-        s = '{}[{}->{}]'.format(self.production.symbol,
-                                self.start_position,
-                                self.end_position)
-        if self.children:
-            for n in self.children:
-                if hasattr(n, 'tree_str'):
-                    s += '\n' + indent + n.tree_str(depth+1)
-                else:
-                    s += '\n' + indent + n.__class__.__name__ \
-                         + '(' + str(n) + ')'
-        return s
-
-    @property
-    def symbol(self):
-        return self.production.symbol
-
-    def __str__(self):
-        return '<NonTerm(start={}, end={}, sym={})>'\
-            .format(self.start_position, self.end_position,
-                    self.production.symbol)
-
-    def __iter__(self):
-        return iter(self.children)
-
-    def __reversed__(self):
-        return reversed(self.children)
-
-
-class NodeTerm(Node):
-    def __init__(self, context):
-        super(NodeTerm, self).__init__(context)
-
-    @property
-    def symbol(self):
-        return self.context.token.symbol
-
-    @property
-    def value(self):
-        return self.context.token.value
-
-    @property
-    def additional_data(self):
-        return self.context.token.additional_data
-
-    def tree_str(self, depth=0):
-        return '{}[{}->{}, "{}"]'.format(self.symbol,
-                                         self.start_position,
-                                         self.end_position,
-                                         self.value)
-
-    def __str__(self):
-        return '<Term(start={}, end={}, sym={}, val="{}")>'\
-            .format(self.start_position, self.end_position, self.symbol,
-                    self.value[:20])
-
-    def __iter__(self):
-        return iter([])
-
-    def __reversed__(self):
-        return iter([])
-
-
 class Token(object):
     """
     Token or lexeme matched from the input.
     """
-    __slots__ = ['symbol', 'value', 'additional_data', 'length']
+    __slots__ = ['symbol', 'value', 'additional_data', 'length', 'position']
 
-    def __init__(self, symbol=None, value='', additional_data=(), length=None):
+    def __init__(self, symbol, value, position, additional_data=(), length=None):
         self.symbol = symbol
         self.value = value
         self.additional_data = additional_data
         self.length = length if length is not None else len(value)
+        self.position = position
 
     def __repr__(self):
         return "<{}({})>".format(str(self.symbol), str(self.value))
@@ -1017,9 +924,12 @@ class Token(object):
     def __len__(self):
         return self.length
 
+    @property
+    def end_position(self):
+        return self.position + self.length
+
     def __bool__(self):
         return True
 
 
-STOP_token = Token(STOP)
-EMPTY_token = Token(EMPTY)
+STOP_token = Token(STOP, '', None)
