@@ -14,4 +14,4 @@ def test_glr_list_building_bug():
     parser = GLRParser(g, prefer_shifts=True)
     result = parser.parse('b a b a a a')
     assert len(result) == 1
-    assert result[0] == [['b', ['a']], ['b', ['a', 'a', 'a']]]
+    assert parser.call_actions(result[0]) == [['b', ['a']], ['b', ['a', 'a', 'a']]]
